var _ = require('lodash');
var React = require('react');

var ConfigGroups = require('./ConfigGroups.jsx');
var ItemStore = require('../services/ItemStore');

var Configure = React.createClass({
  getInitialState: function() {
    ItemStore.setGroups(this.props.groups);
    return {
      groups: this.props.groups
    };
  },

  componentWillReceiveProps: function(nextProps) {
    ItemStore.setGroups(nextProps.groups);
    this.setState({groups: nextProps.groups});
  },

  handleGroupsChange: function(groupName, itemName, value, data) {
    var getValues = function(value) {
      if (!value) {
        return [];
      }
      return _.isArray(value) ? value : [value];
    };
    var getValue = function(value) {
      if (!value) {
        return value;
      }
      return _.isArray(value) ? _.first(value) : value;
    };
    var groups = _.map(this.state.groups, function(group) {
      if (group.name === groupName) {
        group.items = _.map(group.items, function(item) {
          if (!(item.type in ['select_many', 'label', 'heading']) &&
              item.name === itemName) {
            if (item.multiple) {
              item.multi_value = getValues(value);
              if (item.type === 'file') {
                item.multi_data = getValues(data);
              }
            } else {
              item.value = getValue(value);
              if (item.type === 'file') {
                item.data = getValue(data);
              }
            }
          } else {
            if (item.type !== 'select_one') {
              item.items = _.map(item.items, function(childItem) {
                if (childItem.name === itemName) {
                  if (childItem.multiple) {
                    childItem.multi_value = getValues(value);
                    if (childItem.type === 'file') {
                      childItem.multi_data = getValues(data);
                    }
                  } else {
                    childItem.value = getValue(value);
                    if (childItem.type === 'file') {
                      childItem.data = getValue(data);
                    }
                  }
                }
                return childItem;
              });
            }
          }
          return item;
        });
      }
      return group;
    });

    ItemStore.setGroups(groups);
    this.setState({groups: groups});

    // TODO: maybe this should only be on submit
    this.triggerChange(groups);
  },

  triggerChange: function(groups) {
    if (this.props.handleChange) {
      this.props.handleChange(this.getData(groups));
    }
  },

  getData: function(groups) {
    if (!groups) {
      groups = this.state.groups;
    }

    var getItemData = function(item) {
      var data = {
        name: item.name,
        value: '',
        multi_value: []
      };
      if (ItemStore.isEnabled(item)) {
        if (item.multiple) {
          if (item.multi_value && item.multi_value.length) {
            data.multi_value = item.multi_value;
          } else if (item.default) {
            data.multi_value = [item.default];
          }
        } else {
          if (item.value && item.value.length) {
            data.value = item.value;
          } else {
            data.value = item.default;
          }
        }
        if (item.type === 'file') {
          data.data = '';
          if (item.multiple) {
            if (item.multi_data && item.multi_data.length) {
              data.multi_data = item.multi_data;
            } else {
              data.multi_data = [];
            }
          } else {
            data.data = item.data;
          }
        }
      }
      return data;
    };

    var data = [];
    _.each(groups, function(group) {
      _.each(group.items, function(item) {
        if (item.type !== 'select_many') {
          data.push(getItemData(item));
        }
        if (item.type !== 'select_one') {
          _.each(item.items, function(childItem) {
            data.push(getItemData(childItem));
          });
        }
      });
    });

    return data;
  },

  render: function() {
    return (
      <ConfigGroups
        groups={this.state.groups}
        handleChange={this.handleGroupsChange}
        onTestProc={this.props.onTestProc} />
    );
  }
});

module.exports = Configure;
