var _ = require('lodash');

var ItemStore = {
  setGroups: function(groups) {
    this.groups = _.filter(groups, function(group) {
      return !_.isEmpty(group);
    });

    this.items = _.without(_.flatten(_.map(this.groups, function(group) {
      return _.map(_.get(group, 'items', []), function(item) {
        if (!_.isEmpty(item)) {
          return item;
        }
      });
    })), null);
  },

  getItem: function(itemName) {
    var item = null;
    _.some(this.items, function(otherItem) {
      if (otherItem.name === itemName) {
        item = otherItem;
        return true;
      } else {
        return _.some(otherItem.items, function(otherChildItem) {
          if (otherChildItem.name === itemName) {
            item = otherChildItem;
            return true;
          }
          return false;
        });
      }
    });
    return item;
  },

  evaluateWhen: function(when) {
    var checkWhen = function(i, v, ne) {
      var val = _.get(i, 'value', null);
      if (ne) {
        return val && val.length > 0 ? val !== v : i.default !== v;
      } else {
        return val && val.length > 0 ? val === v : i.default === v;
      }
    };

    if (when.length > 0) {
      var whenParts = when.split('=', 2);
      if (whenParts.length === 2) {
        var opNe = false,
          itemName = whenParts[0],
          itemValue = whenParts[1];
        if (itemName.substr(itemName.length - 1) === '!') {
          opNe = true;
          itemName = itemName.substr(0, itemName.length - 1);
        }

        var theItem = this.getItem(itemName);
        if (theItem !== null) {
          return checkWhen(theItem, itemValue, opNe);
        }
      }
    }

    return true;
  },

  isItemVisible: function(item) {
    return !item.hidden && this.isEnabled(item);
  },

  isEnabled: function(obj) {
    var when = _.get(obj, 'when', '')
    return when.length > 0 ? this.evaluateWhen(when) : true;
  },

  getEnabledGroups: function() {
    return _.filter(this.groups, this.isEnabled.bind(this));
  },
};

module.exports = ItemStore;
